@@ -73,9 +73,11 @@ dates_date_change (DatesData *data, int direction)
     else
 	icaltime_adjust (time, direction, 0, 0, 0);
 	
+    if (time->year < 1900)
+        goto out;
     dates_view_set_date (data->view, time);
 	
-    g_free (time);
+out:    g_free (time);
 }
 
 void
@@ -408,6 +410,7 @@ dates_fill_details_dialog (DatesView *view, DatesData *d)
     GtkTextBuffer *buffer;
     ECalComponentText summary;
     GtkWidget *widget;
+    icaltimezone* from_zone; 
 
     /* Set summary entry */
     e_cal_component_get_summary (d->comp, &summary);
@@ -445,13 +448,15 @@ dates_fill_details_dialog (DatesView *view, DatesData *d)
 	
     /* Set time buttons and dialog */
     e_cal_component_get_dtstart (d->comp, &time);
-    timem = icaltimetype_to_tm (time.value);
+    from_zone = icaltimezone_get_builtin_timezone_from_tzid (time.tzid);
+    timem = icaltimetype_to_tm_with_zone (time.value, from_zone, dates_view_get_zone(view));
     strftime (time_text, 256, TIME_MARKUP, &timem);
     gtk_label_set_markup (GTK_LABEL (d->details_start_label), time_text);
     e_cal_component_free_datetime (&time);
 
     e_cal_component_get_dtend (d->comp, &time);
-    timem = icaltimetype_to_tm (time.value);
+    from_zone = icaltimezone_get_builtin_timezone_from_tzid (time.tzid);
+    timem = icaltimetype_to_tm_with_zone (time.value, from_zone, dates_view_get_zone(view));
     strftime (time_text, 256, TIME_MARKUP, &timem);
     gtk_label_set_markup (GTK_LABEL (d->details_end_label),
 			  time_text);
@@ -1052,12 +1057,48 @@ dates_details_ok_cb (GtkWidget *source, DatesData *d)
     }
 }
 
+static char *
+str_truncate (const char *long_str, guint max_length)
+{
+  char *p;
+  char *p_end;
+  const char *ellipsis_text;
+
+  if (max_length == 0)
+    return NULL;
+
+  if (!g_utf8_validate (long_str, -1, NULL))
+    return NULL;
+
+  p = g_strdup (long_str);
+
+  if (g_utf8_strlen (p, -1) < max_length)
+    return p;
+
+  /* 3 characters so that we definitely have space for ellipsis */
+  p_end = g_utf8_offset_to_pointer (p, max_length - 3);
+
+  /* Write an ellipsis */
+  if (g_unichar_iswide (g_utf8_get_char (p)))
+    ellipsis_text = "\342\213\257"; /* U+22EF: MIDLINE HORIZONTAL ELLIPSIS, used for CJK */
+  else
+    ellipsis_text = "\342\200\246"; /* U+2026: HORIZONTAL ELLIPSIS */
+  g_utf8_strncpy (p_end, ellipsis_text, 1);
+
+  /* Terminate the string */
+  p_end = g_utf8_next_char (p_end);
+  *p_end = '\0';
+
+  return p;
+}
+
 void
 dates_delete_cb (GtkWidget *source, DatesData *d)
 {
     ECalComponentText summary;
     GtkWidget *widget;
-    const char *value;
+    const char *orig_value;
+    char *value;
 #ifdef WITH_HILDON
     gchar *tmp;
 #endif
@@ -1065,16 +1106,18 @@ dates_delete_cb (GtkWidget *source, DatesData *d)
     e_cal_component_get_summary (d->comp, &summary);
     widget = d->details_dialog;
     if (GTK_WIDGET_VISIBLE (widget))
-	value = gtk_entry_get_text (GTK_ENTRY (d->details_summary_entry));
+	orig_value = gtk_entry_get_text (GTK_ENTRY (d->details_summary_entry));
     else if (summary.value)
-	value = summary.value;
+	orig_value = summary.value;
     else if (summary.altrep)
-	value = summary.altrep;
+	orig_value = summary.altrep;
     else {
 	g_warning ("Deleting event with no summary");
-	value = _("Unknown event");
+	orig_value = _("Unknown event");
     }
-		
+
+    value = str_truncate (orig_value, 50);
+
 #ifdef WITH_HILDON
     tmp = g_strdup_printf(_("Are you sure you want to delete event '%s'?"), value);
     widget = hildon_note_new_confirmation_add_buttons(
@@ -1118,6 +1161,7 @@ dates_delete_cb (GtkWidget *source, DatesData *d)
     }
 	
     gtk_widget_destroy (widget);
+    g_free (value);
 }
 
 void
@@ -1775,6 +1819,17 @@ dates_import_calendar_data_from_file (ECal *cal, gchar *filename, GError **error
     GIOStatus status;
     icalparser *parser = icalparser_new ();
 
+    /* set the channel as binary mode and let icalparser_add_line
+     * handle encoding */
+    g_io_channel_set_encoding (channel, NULL, &tmp_error);
+    if (tmp_error != NULL)
+    {
+      g_warning ("Error when set encoding: %s", tmp_error->message);
+      g_propagate_error (error, tmp_error);
+      g_io_channel_unref (channel);
+      return FALSE;
+    }
+
     /* Read the from the file line by line and until EOF */
     while ((status = g_io_channel_read_line (channel, &line, &length, NULL, &tmp_error))
         == G_IO_STATUS_NORMAL)
